/*
 * GuiDes source code:
 *
 * Copyright 2002 Carlos Andrs <carlos.andres@navegalia.com>
 * www - http://www.geocities.com/harbour_links/guides
 *
 * GuiDes is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * GuiDes is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA (or visit the web site http://www.gnu.org/).
 *
 */

#include "GUIDES.CH"
#include "minigui.ch"
#include "hbclass.ch"
#include "Fileio.ch"

CREATE CLASS gTextEditor

   DATA FileName		INIT ""
   DATA ReadOnly		INIT FALSE
   DATA Text		INIT ""

	METHOD New()               CONSTRUCTOR   
	METHOD Save()
	METHOD Compile()
	METHOD HasChanged()
	METHOD GetFileName()
	METHOD GetFileText()
	METHOD Open()
END CLASS


METHOD New( cFileName, lReadOnly ) CLASS gTextEditor
   LOCAL FinalText := ""

	::FileName := cFileName
	::ReadOnly := lReadOnly

	IF EMPTY( ::FileName )	
      	::FileName := ::GetFileName()
	ENDIF	

   	IF ::ReadOnly == NIL
      	::ReadOnly := TRUE
   	ENDIF

	::Text := ::GetFileText()
    
      DEFINE WINDOW TextFileWin ;
              AT 50,200 ; 
              WIDTH 610 ; 
              HEIGHT 545 ; 
              TITLE ::FileName  ; 
              ICON "GUIDES" ;
              MODAL ;
              ON RELEASE ::Save( FALSE )
                              
              IF ::ReadOnly
                  @ 0,0 EDITBOX Texto ; 
                  WIDTH 600 HEIGHT 500 ; 
                  VALUE ::Text ; 
                  READONLY  ; 
                  FONT 'Courier New' SIZE 10 ;
                  BACKCOLOR G_LIGHTGREY
              ELSE

        		DEFINE MAIN MENU

		            POPUP 'File'

            		    ITEM 'Open' ; 
		                ACTION ::Open( )  

            		    ITEM 'Save' ; 
		                ACTION ::Save( TRUE )  
				
            		    ITEM 'Compile' ; 
		                ACTION ::Compile()  

				    ITEM 'Exit' ;
		                ACTION ThisWindow.Release  

            		END POPUP

			END MENU

		DEFINE SPLITBOX

   			DEFINE TOOLBAR ToolBar1 BUTTONSIZE 40,40 FONT "ARIAL" SIZE 8 FLAT 

		         BUTTON tbOpen ;
		         PICTURE 'LOADWIN' ;
		         ACTION ::Open()

		         BUTTON tbSave ;
		         PICTURE 'SAVE' ;
		         ACTION ::Save( TRUE )

		         BUTTON tbcOMPILE ;
		         PICTURE 'COMPILE' ;
		         ACTION ::Compile( )

			END TOOLBAR
	
                  EDITBOX Texto ;
                  WIDTH 1000 HEIGHT 420 ; 
                  VALUE ::Text ; 
                  FONT 'Courier New' SIZE 10  
			
		END SPLITBOX

              ENDIF

              DEFINE STATUSBAR
            
                  STATUSITEM IIF( ::ReadOnly, "READONLY", "EDITABLE" ) WIDTH 100 
            
              END STATUSBAR
         END WINDOW

         CENTER WINDOW TextFileWin
         ACTIVATE WINDOW TextFileWin 
RETURN NIL


METHOD Save( lForce ) CLASS gTextEditor
	LOCAL FinalText := ""

	IF !lForce .AND. ::HasChanged()
		lForce := MsgYesNo( ::FileName + " has changed" + CR_LF + "Do you want to save changes now?", "GuiDes" )
	ENDIF		

	IF lForce
		cFinalText := TextFileWin.Texto.Value
		MEMOWRIT( ::FileName, cFinalText )
		MsgBox( ::FileName + " has been saved", "GuiDes" )
	ENDIF
RETURN NIL


METHOD Compile() CLASS gTextEditor
	LOCAL cCompiler := oStyle:CompileFile()
	LOCAL cFlags   := oStyle:CompileFlags()
      LOCAL cNoExtensionFileName := ""
	LOCAL lForce := TRUE
	LOCAL cCompilation := ""

	IF EMPTY( ALLTRIM(cCompiler) )
		MsgBox( "No compiler selected!", "GuiDes" )
	ELSE
		IF !FILE( cCompiler )
			MsgBox( "Cannot find compiler executable", "GuiDes" )
		ELSE
			IF ::HasChanged()
				IF MsgYesNo( ::FileName + " has changed" + CR_LF + "Do you want to save changes before compilation?" ;
						+ CR_LF + CR_LF + "(Compilation is done on the last saved version of the file)", "GuiDes" )
					::Save( TRUE )
				ENDIF
			ENDIF
			cNoExtensionFileName := RemoveExtension( ::FileName )
			cCompilation := cCompiler + " " + cNoExtensionFileName + " " + ALLTRIM( cFlags )
			EXECUTE FILE cCompilation WAIT
			MsgBox( "Compilation finished", "GuiDes" )
		ENDIF			
	ENDIF
RETURN NIL


METHOD HasChanged() CLASS gTextEditor
	LOCAL FinalText := TextFileWin.Texto.Value

RETURN !( FinalText == ::Text )



METHOD GetFileName() CLASS gTextEditor
	LOCAL acFilter := { { "Harbour source files (*.prg)", "*.PRG" }, ;
                    { "Guides files (*.frm)", "*.FRM" }, ;
                    { "ASCII files (*.txt)", "*.TXT" }, ;
                    { "All files (*.*)", "*.*" } }

	LOCAL cFileName := ""
   
      cFileName := GetFile( acFilter, , oStyle:ProjectDir() )

RETURN cFileName


METHOD GetFileText() CLASS gTextEditor
	LOCAL cText := ""

   	IF !EMPTY( ::FileName )
      	IF FILE( ::FileName )
         		cText := MEMOREAD( ::FileName )
   	      ELSE
         		MsgBox("ERROR! File " + CR_LF + ::FileName + CR_LF + "not found!", "GuiDes")
      	ENDIF
	ENDIF
RETURN cText


METHOD Open() CLASS gTextEditor()
	LOCAL cFileName := ::GetFileName()

	IF !EMPTY( cFileName )
		::Save( FALSE )
		::FileName := cFileName
		::Text := ::GetFileText()
		TextFileWin.Title := "GuiDes Text Editor: " + cFileName
		TextFileWin.Texto.Value := ::Text
	ENDIF

RETURN NIL

